@@ -1,13 +1,12 @@
 import pandas as pd
 import logging
+import tqdm
 
 from fhir2dataset.graphquery import GraphQuery
 from fhir2dataset.fhirrules_getter import FHIRRules
 from fhir2dataset.api_caller import ApiGetter
 from fhir2dataset.url_builder import URLBuilder
 from fhir2dataset.graph_tools import join_path
-from fhir2dataset.timer import timing
-
 
 logger = logging.getLogger(__name__)
 
@@ -16,7 +15,7 @@ class Query:
     """Query Executor
 
     Description:
-        This module purpose is to perform SQL-type queries whose information is 
+        This module purpose is to perform SQL-type queries whose information is
         filled in configuration files of the form:
         ```
         {
@@ -28,12 +27,12 @@ class Query:
             },
             "select": {
                 "alias n°1": [
-                    "expression of attribute a of resource type 1",
-                    "expression of attribute b of resource type 1",
-                    "expression of attribute c of resource type 1"
+                    "searchparam/fhirpath of attribute a of resource type 1",
+                    "searchparam/fhirpath of attribute b of resource type 1",
+                    "searchparam/fhirpath of attribute c of resource type 1"
                 ],
                 "alias n°2": [
-                    "expression of attribute a of resource type 2",
+                    "searchparam of attribute a of resource type 2",
                     ...
                 ],
                 ...
@@ -71,21 +70,21 @@ class Query:
         for the next associated SQL query:
         ```
         SELECT (alias n°1).a, (alias n°1).b, (alias n°1).c, (alias n°2).a FROM (Resource type 1) as (alias n°1)
-        INNER JOIN (Resource type 2) as (alias n°2) 
-        ON (alias n°1).d = (alias n°2) 
-        INNER JOIN (Resource type 3) as (alias n°3) 
-        ON (alias n°2).b = (alias n°3) WHERE (alias n°2).c = "value 1"  
-        AND (alias n°2).d = "value 2"  
-        AND (alias n°3).a = "value 3"  
+        INNER JOIN (Resource type 2) as (alias n°2)
+        ON (alias n°1).d = (alias n°2)
+        INNER JOIN (Resource type 3) as (alias n°3)
+        ON (alias n°2).b = (alias n°3) WHERE (alias n°2).c = "value 1"
+        AND (alias n°2).d = "value 2"
+        AND (alias n°3).a = "value 3"
         AND (alias n°3).b = "value 4"
     ```
-    
+
     Attributes:
         config {dict} -- dictionary storing the initial request
         graph_query {type(GraphQuery)} -- instance of a GraphQuery object that gives a graphical representation of the query
         dataframes {dict} -- dictionary storing for each alias the resources requested on the api in tabular format
         main_dataframe {DataFrame} -- pandas dataframe storing the final result table
-    
+
     Example:
         import fhir2dataset
         import logging
@@ -97,9 +96,13 @@ class Query:
         df = query.main_dataframe
     """  # noqa
 
-    @timing
-    def __init__(self, fhir_api_url: str, fhir_rules: type(FHIRRules) = None, token: str = None):
-        """Requestor's initialisation 
+    def __init__(
+        self,
+        fhir_api_url: str = None,
+        fhir_rules: type(FHIRRules) = None,
+        token: str = None,
+    ):
+        """Requestor's initialisation
 
         Arguments:
             fhir_api_url {str} -- The Service Base URL (e.g. http://hapi.fhir.org/baseR4/)
@@ -108,7 +111,7 @@ class Query:
             fhir_rules {type(FHIRRules)} -- an instance of a FHIRRules-type object. If the instance is not filled a default version will be used. (default: {None})
             token {str} -- bearer token authentication if necessary (default: {None})
         """  # noqa
-        self.fhir_api_url = fhir_api_url
+        self.fhir_api_url = fhir_api_url or "http://hapi.fhir.org/baseR4/"
         if not fhir_rules:
             fhir_rules = FHIRRules(fhir_api_url=self.fhir_api_url)
         self.fhir_rules = fhir_rules
@@ -116,10 +119,9 @@ class Query:
 
         self.config = None
         self.graph_query = None
-        self.dataframes = dict()
+        self.dataframes = {}
         self.main_dataframe = None
 
-    @timing
     def from_config(self, config: dict):
         """Executes the query from a dictionary in the format of a configuration file
 
@@ -132,8 +134,8 @@ class Query:
             "where_dict": config.get("where", None),
             "join_dict": config.get("join", None),
         }
+        return self
 
-    @timing
     def execute(self, debug: bool = False):
         """Executes the complete query
 
@@ -147,28 +149,22 @@ class Query:
         """  # noqa
         self.graph_query = GraphQuery(fhir_api_url=self.fhir_api_url, fhir_rules=self.fhir_rules)
         self.graph_query.execute(**self.config)
-        for resource_alias in self.graph_query.resources_alias_info.keys():
-            resource_alias_info = self.graph_query.resources_alias_info[resource_alias]
-            elements = resource_alias_info["elements"]
-            elements_concat_type = resource_alias_info["elements_concat_type"]
-
-            url_builder = URLBuilder(
-                fhir_api_url=self.fhir_api_url,
-                query_graph=self.graph_query,
-                main_resource_alias=resource_alias,
-            )
-
-            url = url_builder.search_query_url
-            call = ApiGetter(
-                url=url,
-                elements=elements,
-                elements_concat_type=elements_concat_type,
-                main_resource_alias=resource_alias,
-                token=self.token,
-            )
-            call.get_all()
-            self.dataframes[resource_alias] = call.display_data()
-            # print(elements)
+        with tqdm.tqdm(total=1000) as pbar:
+            time_frac = round(1000 / len(self.graph_query.resources_alias_info))
+            for resource_alias in self.graph_query.resources_alias_info.keys():
+                resource_alias_info = self.graph_query.resources_alias_info[resource_alias]
+                elements = resource_alias_info.elements
+                url_builder = URLBuilder(
+                    fhir_api_url=self.fhir_api_url,
+                    graph_query=self.graph_query,
+                    main_resource_alias=resource_alias,
+                )
+                url = url_builder.compute()
+                call = ApiGetter(
+                    url=url, elements=elements, token=self.token, pbar=pbar, time_frac=time_frac
+                )
+                call.get_all()
+                self.dataframes[resource_alias] = call.df
         self._clean_columns()
         for resource_alias, dataframe in self.dataframes.items():
             logger.debug(f"{resource_alias} dataframe builded head - \n{dataframe.to_string()}")
@@ -184,22 +180,35 @@ class Query:
         )
         if not debug:
             self._select_columns()
-        # to do check where
+            self._remove_lists()
 
-    @timing
     def _select_columns(self):
-        """Clean the final dataframe to keep only the columns of the select
-        """
+        """Clean the final dataframe to keep only the columns of the select"""
         final_columns = []
-        for resource_alias in self.graph_query.resources_alias_info.keys():
-            resource_alias_info = self.graph_query.resources_alias_info[resource_alias]
-            elements_select = resource_alias_info["elements"]["select"]
-            final_columns.extend([f"{resource_alias}:{element}" for element in elements_select])
+        for resource_alias, resource_alias_info in self.graph_query.resources_alias_info.items():
+            for element in resource_alias_info.elements.get_subset_elements(goal="select"):
+                final_columns.append(f"{resource_alias}:{element.col_name}")
         self.main_dataframe = self.main_dataframe[final_columns]
 
-    @timing
+    def _remove_lists(self):
+        """Remove lists from columns with only single elements"""
+        df = self.main_dataframe
+        for column in df.columns:
+            lengths = df[column].str.len()
+            try:
+                if all(len < 2 for len in lengths):
+                    df[column] = df[column].apply(lambda x: x[0] if isinstance(x, list) else x)
+                    logger.info(
+                        f"extraction of the unique element from the lists composing "
+                        f"the {column} column"
+                    )
+            except Exception:
+                pass
+        self.main_dataframe = df
+
     def _join(self) -> pd.DataFrame:
-        """executes the joins one after the other in the order specified by the join_path function.
+        """Execute the joins one after the other in the order specified by the
+        join_path function.
 
         Returns:
             pd.DataFrame -- dataframe containing all joined resources
@@ -213,13 +222,11 @@ class Query:
             main_df = self._join_2_df(alias_1, alias_2, df_1, df_2)
         return main_df
 
-    @timing
     def _group_lines(self, df, col_name):
         logger.debug(f"dataframe before being grouped \n{df.to_string()}")
         if not df.empty:
             cols_group = [col_name]
             if cols_group:
-                # cols_group += self.elements['additional_resource']
                 cols = df.columns.to_list()
                 cols_list = [col_name for col_name in cols if col_name not in cols_group]
                 dict_cols_list = {col: self._concatenate for col in cols_list}
@@ -228,26 +235,24 @@ class Query:
             logger.debug(f"dataframe after being grouped on {cols_group}:\n{df.to_string()}")
         return df
 
-    @timing
     def _concatenate(self, column):
         result = []
         for list_cell in column:
             if isinstance(list_cell, list):
-                result.extend([value for value in list_cell])
+                result.extend(list_cell)
             else:
                 result.append(list_cell)
         return result
 
-    @timing
     def _join_2_df(
-        self, alias_1: str, alias_2: str, df_1: pd.DataFrame, df_2: pd.DataFrame,
+        self, alias_1: str, alias_2: str, df_1: pd.DataFrame, df_2: pd.DataFrame
     ) -> pd.DataFrame:
         """Executes the join between two dataframes
-        
+
         The join key is the id of the child resource.
         This id is contained in :
             * in the column child_alias:id of the child resource table named child_alias (e.g. parent:id for a parent dataframe)
-            * in the alias_parent:element_join column of the parent resource table named alias_parent (e.g. condition:subject.reference for a condition dataframe)
+            * in the alias_parent:searchparam_parent column of the parent resource table named alias_parent (e.g. condition:subject.reference for a condition dataframe)
 
         The function is in charge of finding out who is the mother resource and who is the daughter resource.
 
@@ -259,15 +264,15 @@ class Query:
         Returns:
             pd.DataFrame -- dataframe containing the elements of the 2 resources according to an inner join
         """  # noqa
-        edge_info = self.graph_query.resources_alias_graph.edges[alias_1, alias_2]
-        alias_parent = edge_info["parent"]
-        alias_child = edge_info["child"]
+        edge_info = self.graph_query.resources_alias_graph.edges[alias_1, alias_2]["info"]
+        alias_parent = edge_info.parent
+        alias_child = edge_info.child
 
-        how = edge_info["join_how"]
-        element_join = edge_info["element_join"]
+        how = edge_info.join_how
+        searchparam_parent = edge_info.searchparam_parent
 
-        parent_on = f"{alias_parent}:{element_join}"
-        child_on = f"{alias_child}:id"
+        parent_on = f"{alias_parent}:join_{searchparam_parent}"
+        child_on = f"{alias_child}:from_id"
 
         if how == "child":
             how = "right"
@@ -277,51 +282,22 @@ class Query:
             how = "inner"
 
         if alias_1 == alias_parent:
-            ### to delete after ??  # noqa
-            if alias_1 == "patient":
-                df_2 = self._group_lines(df_2, child_on)
-            elif alias_2 == "patient":
-                df_1 = self._group_lines(df_1, parent_on)
-            ##################  # noqa
             df_merged_inner = pd.merge(
-                left=df_1, right=df_2, left_on=parent_on, right_on=child_on, how=how,
+                left=df_1, right=df_2, left_on=parent_on, right_on=child_on, how=how
             )
         else:
-            ### to delete after ??  # noqa
-            if alias_1 == "patient":
-                df_2 = self._group_lines(df_2, parent_on)
-            elif alias_2 == "patient":
-                df_1 = self._group_lines(df_1, child_on)
-            ###################  # noqa
             df_merged_inner = pd.merge(
-                left=df_2, right=df_1, left_on=parent_on, right_on=child_on, how=how,
+                left=df_2, right=df_1, left_on=parent_on, right_on=child_on, how=how
             )
         return df_merged_inner
 
-    @timing
-    def _get_main_alias_join(self) -> str:
-        """returns the alias being involved in the maximum number of joins
-
-        Returns:
-            str -- an alias
-        """
-        main_alias = None
-        max_join = -1
-        for (alias, infos,) in self.graph_query.resources_alias_info.items():
-            num_join = len(infos["elements"]["join"])
-            if max_join < num_join:
-                main_alias = alias
-                max_join = num_join
-        return main_alias
-
-    @timing
     def _clean_columns(self):
         """performs preprocessing on all dataframes harvested in the dataframe attribute:
-            * adds the resource type in front of an element id so that the resource id matches the references of its parent resource references
-            * adds the table alias as a prefix to each column name
+        * adds the resource type in front of an element id so that the resource id matches the references of its parent resource references
+        * adds the table alias as a prefix to each column name
         """  # noqa
         for resource_alias, df in self.dataframes.items():
-            resource_type = self.graph_query.resources_alias_info[resource_alias]["resource_type"]
+            resource_type = self.graph_query.resources_alias_info[resource_alias].resource_type
 
             df = df.pipe(_add_resource_type_to_id, resource_type=resource_type)
 
@@ -330,5 +306,5 @@ class Query:
 
 def _add_resource_type_to_id(df, resource_type: str):
     # add assert to check that there is only one id in list
-    df["id"] = f"{resource_type}/" + df["id"]
+    df["from_id"] = f"{resource_type}/" + df["from_id"]
     return df
